//-----------------------------------------------------------------
// File : asdxDemoCameraImpl.cpp
// Desc : Asura Camera For Demo Application
// Copyright(c) Project Asura. All right reserved.
//-----------------------------------------------------------------

//-----------------------------------------------------------------
// History
//-----------------------------------------------------------------
// <---> 2011/08/22 [Pocol] VK쐬
// <End>

//-----------------------------------------------------------------
// Includes
//-----------------------------------------------------------------
#include "asDef.h"
#include "asdxDemoCameraImpl.h"


namespace Asura {
namespace SampleProgram {

//////////////////////////////////////////////////////////////////
// DemoCameraImpl class
//////////////////////////////////////////////////////////////////

//----------------------------------------------------------------
//! @brief lݒ
//----------------------------------------------------------------
DemoCameraImpl::DemoCameraImpl()
{
    mPosition  = D3DXVECTOR3( 0.0f, 0.0f, 1.0f );
    mDirection = D3DXVECTOR3( 0.0f, 0.0f, 0.0f );
    mUp        = D3DXVECTOR3( 0.0f, 1.0f, 0.0f );

    mAngle = D3DXVECTOR2( (float)D3DXToRadian( 45.0f ), (float)D3DXToRadian( 45.0f ) );
    mDistance = 5.0f;

    std::memset( &mLeft, 0, sizeof( MouseState ) );
    std::memset( &mRight, 0, sizeof( MouseState ) );
    std::memset( &mMiddle, 0, sizeof( MouseState ) );
    std::memset( &mCbValue, 0, sizeof( mCbValue ) );

    D3DXMatrixIdentity( &mView );
}

//---------------------------------------------------------------
//! @brief j
//---------------------------------------------------------------
DemoCameraImpl::~DemoCameraImpl()
{ /* DO_NOTHING */ }

//---------------------------------------------------------------
//! @brief J̈ʒuݒ肷
//---------------------------------------------------------------
void DemoCameraImpl::SetPosition( D3DXVECTOR3 position )
{
    float dist = D3DXVec3Length( &position );
    mDistance = dist;

    D3DXVECTOR3 xAxis( 1.0f, 0.0f, 0.0f );
    D3DXVECTOR3 yAxis( 0.0f, 1.0f, 0.0f );

    D3DXVECTOR3 normPos = position;
    D3DXVec3Normalize( &normPos, &normPos );
    float angleX = D3DXVec3Dot( &xAxis, &normPos );
    float angleY = D3DXVec3Dot( &yAxis, &normPos );

    mAngle.x = angleX;
    mAngle.y = angleY;

    mPosition = position;
}

//---------------------------------------------------------------
//! @brief J̕ݒ肷
//---------------------------------------------------------------
void DemoCameraImpl::SetDirection( D3DXVECTOR3 direction )
{ mDirection = direction; }

//---------------------------------------------------------------
//! @brief J̃AbvxNgݒ肷
//---------------------------------------------------------------
void DemoCameraImpl::SetUpVector( D3DXVECTOR3 up )
{ mUp = up; }

//---------------------------------------------------------------
//! @brief J̈ʒu擾
//---------------------------------------------------------------
D3DXVECTOR3 DemoCameraImpl::GetPosition()
{ return mPosition; }

//---------------------------------------------------------------
//! @brief J̕擾
//---------------------------------------------------------------
D3DXVECTOR3 DemoCameraImpl::GetDirection()
{ return mDirection; }

//---------------------------------------------------------------
//! @brief J̃AbvxNg擾
//---------------------------------------------------------------
D3DXVECTOR3 DemoCameraImpl::GetUpVector()
{ return mUp; }

//---------------------------------------------------------------
//! @brief J̃r[s擾
//---------------------------------------------------------------
D3DXMATRIX DemoCameraImpl::GetView()
{ return mView; }

//---------------------------------------------------------------
//! @brief J̃r[svZ
//---------------------------------------------------------------
void DemoCameraImpl::Compute()
{
    ComputeAsThirdPersonCamera();
}

//---------------------------------------------------------------
//! @brief }EX
//---------------------------------------------------------------
void DemoCameraImpl::OnMouse
(
    bool isLeftDown,
    bool isRightDown,
    bool isMiddleDown,
    bool isSide1Down,
    bool isSide2Down,
    int  wheelDelta,
    int  x,
    int  y
)
{
    mCbValue.cursorX = x;
    mCbValue.cursorY = y;

    mCbValue.wheelDelta   = wheelDelta;
    mCbValue.isDownLeft   = isLeftDown;
    mCbValue.isDownRight  = isRightDown;
    mCbValue.isDownMiddle = isMiddleDown;
    mCbValue.isDownSide1  = isSide1Down;
    mCbValue.isDownSide2  = isSide2Down;
}

//------------------------------------------------------------------
//! @brief 3l̃JƂăr[svZ
//------------------------------------------------------------------
void DemoCameraImpl::ComputeAsThirdPersonCamera()
{
    if ( mCbValue.isDownLeft ) 
    {
        POINT point;
        GetCursorPos( &point );
        int cursorX = point.x;
        int cursorY = point.y;

        if ( !mLeft.isDrag )
        {
            mLeft.isDrag = true;

            mLeft.cursorX = cursorX;
            mLeft.cursorY = cursorY;
        }

        int deltaX = cursorX - mLeft.cursorX;
        int deltaY = cursorY - mLeft.cursorY;

        mLeft.cursorX = cursorX;
        mLeft.cursorY = cursorY;

        mAngle.x += static_cast< float >( D3DXToRadian( deltaX ) );
        mAngle.y += static_cast< float >( D3DXToRadian( deltaY ) );

        if ( mAngle.x > 2.0f * D3DX_PI )
        { mAngle.x -= static_cast< float >( 2.0f * D3DX_PI ); }
        else if ( mAngle.x < 0.0f )
        { mAngle.x += static_cast< float >( 2.0f * D3DX_PI ); }

        if ( mAngle.y > 2.0f * D3DX_PI ) 
        { mAngle.y -= static_cast< float >( 2.0f * D3DX_PI ); }
        else if ( mAngle.y < 0.0f )
        { mAngle.y += static_cast< float >( 2.0f * D3DX_PI ); }
    }
    else
    {
        mLeft.isDrag = false;
    }

    if ( mCbValue.isDownRight )
    {
        POINT point;
        GetCursorPos( &point );
        int cursorX = point.x;
        int cursorY = point.y;

        if ( !mRight.isDrag )
        {
            mRight.isDrag = true;
            
            mRight.cursorX = cursorX;
            mRight.cursorY = cursorY;
        }

        int deltaX = cursorX - mRight.cursorX;
        int deltaY = cursorY - mRight.cursorY;

        mRight.cursorX = cursorX;
        mRight.cursorY = cursorY;

        const float speed = 0.1f;

        mDistance += ( deltaY ) * speed;

        if ( mDistance < FLT_EPSILON )
        { mDistance = FLT_EPSILON; }
    }
    else
    {
        mRight.isDrag = false;
    }

    mPosition.x = sinf( mAngle.x ) * cosf( mAngle.y );
    mPosition.y = sinf( mAngle.y );
    mPosition.z = cosf( mAngle.x ) * cosf( mAngle.y );

    mPosition *= mDistance;

    if ( mAngle.y >= D3DXToRadian( 90.0f ) && mAngle.y <= D3DXToRadian( 270.0f ) )
    { mUp = D3DXVECTOR3( 0.0f, -1.0f, 0.0f ); }
    else
    { mUp = D3DXVECTOR3( 0.0f, 1.0f, 0.0f ); }

    D3DXMatrixLookAtLH( &mView, &mPosition, &mDirection, &mUp );
}


} // namespace SampleProgram
} // namespace Asura